package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesConfigService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesInputDefectRecordService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesPartInspectionInputModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesPartInspectionViewModel;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspectionDetail;
import cn.estsh.i3plus.pojo.mes.bean.rework.MesReworkTask;
import cn.estsh.i3plus.pojo.mes.bean.rework.MesReworkTaskDetail;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-05-09 10:54
 * @Modify:
 **/

@Service
@Slf4j
public class MesInputDefectRecordService implements IMesInputDefectRecordService {

    @Autowired
    private MesProduceSnRepository produceSnRepository;

    @Autowired
    private MesPartRepository partRepository;

    @Autowired
    private MesPartTypePictureRepository partTypePictureRepository;

    @Autowired
    private MesLocationConfigRepository locationConfigRepository;
    @Autowired
    private MesDefectWarnConfigRepository defectWarnConfigRepository;
    @Autowired
    private MesPartTypeDefectRepository partTypeDefectRepository;
    @Autowired
    private MesMoveRepository moveRepository;

    @Autowired
    private MesPartInspectionRepository partInspectionRepository;
    @Autowired
    private MesPartInspectionDetailRepository partInspectionDetailRepository;

    @Autowired
    private MesReworkTaskRepository reworkTaskRepository;

    @Autowired
    private IMesConfigService configService;

    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private MesDefectTypeRepository defectTypeRepository;

    @Autowired
    private MesReworkTaskDetailRepository mesReworkTaskDetailRDao;

    @Autowired
    private IMesProduceSnExtService produceSnExtService;

    @Autowired
    private MesProductVersionRepository productVersionRepository;

    @Override
    public MesPartInspectionViewModel queryPartInspection(MesPartInspectionInputModel inputModel, String org) {

        //检查物料
        MesPart part;
        //获取NC-零件检测-单据
        MesPartInspection partInspection;
        Integer sourceType = null;
        if (!StringUtils.isEmpty(inputModel.getSn())) {
            // 数据校验
            ValidatorBean.checkNotNull(inputModel.getFrontBack(), "正反面不能为空");
            //检查条码
            MesProduceSn produceSn = checkProduceSn(inputModel.getSn(), org);

            //获取NC-零件检测-单据
            partInspection = getPartInspection(inputModel, org);

            //检查物料
            part = checkPart(produceSn, org);

            sourceType = MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.SINGLE.getValue();

        } else {
            // 数据校验
            ValidatorBean.checkNotNull(inputModel.getPartNo(), "物料不能为空");
            ValidatorBean.checkNotNull(inputModel.getLotNo(), "批次不能为空");
            ValidatorBean.checkNotNull(inputModel.getFrontBack(), "正反面不能为空");
            ValidatorBean.checkNotNull(inputModel.getQty(), "数量不能为空");

            //检查物料
            part = checkPartByPartNo(inputModel.getPartNo(), org);

            //获取NC-零件检测-单据
            partInspection = getPartInspectionByPartAndLot(inputModel, org);

            sourceType = MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.LOT.getValue();
        }

        //检查零件类型与图片对应关系
        MesPartTypePicture partTypePicture = checkPartTypePicture(part, org);

        //构造返回信息
        MesPartInspectionViewModel model = getModel(partInspection, part, partTypePicture, inputModel, sourceType, org);
        model.setSourceType(sourceType);
        model.setTransferFlg(inputModel.isTransferFlg());

        return model;
    }

    /**
     * 查询零件类型缺陷类型对应关系
     *
     * @param partNo
     * @param defectLocation
     * @param flg
     * @param frontBack
     * @param org
     * @return
     */
    @Override
    public List<MesPartTypeDefect> queryPartTypeDefect(String partNo, String defectLocation, boolean flg, Integer frontBack, String org) {

        //检查物料
        MesPart part = checkPartByPartNo(partNo, org);

        //查询零件类型缺陷类型对应关系
        List<MesPartTypeDefect> partTypeDefectList = checkPartTypeDefect(part, org);

        //flg==true 则是标黄的位置，需要查询位置下具体需要标黄的缺陷类型
        if (!StringUtils.isEmpty(flg) && flg) {

            List<String> partTypeDefectCodeList = partTypeDefectList.stream().map(k -> k.getDefectTypeCode()).collect(Collectors.toList());

            //查询预警数量
            List<MesDefectWarnConfig> alarmConfigList = queryDefectAlarmConfig(part, frontBack, defectLocation, partTypeDefectCodeList, org);

            //根据不良类型 合计预警数量
            Map<String, MesDefectWarnConfig> alarmConfigMap = alarmConfigList.stream().collect(Collectors.toMap(k -> k.getDefectTypeCode(), k -> k));

            //位置是否标黄Map,key：不良类型，value：标黄标识
            Map<String, Boolean> map = new HashMap<>();
            alarmConfigMap.forEach((k, v) -> {
                //若有这个不良类型则标黄
                if (!map.containsKey(k)) {
                    map.put(k, true);
                }
            });

            //找到标黄的缺陷类型 标识置为true
            partTypeDefectList.forEach(k -> {
                if (map.containsKey(k.getDefectTypeCode())) {
                    k.setFlg(true);
                }
            });
        }

        return partTypeDefectList;
    }

    /**
     * 完成质检-保存不良记录
     *
     * @param model
     * @param org
     */
    @Override
    public void savePartInspection(MesPartInspectionInputModel model, String org) {


        //零件检测详情为空，则代表本次校验为合格。
        if (CollectionUtils.isEmpty(model.getPartInspectionDetailList())) {
            MesPartInspection partInspection;
            //返工单返修完成，需可疑品移正常库

            if (model.isTransferFlg()) {
                String srcLocate = configService.getCfgValue(org, "UMLGO");
                if(!Objects.isNull(model.getPartInspection())){
                    MesMove mesMove = getMesMove(model, org);
                    if(!Objects.isNull(mesMove)){
                        srcLocate = mesMove.getUmlgo();
                    }
                }
                //移库
                MesMove move = createMove(model,srcLocate,configService.getCfgValue(org, "LGORT"), org);
                moveRepository.save(move);
            }

            //判断首次合格还是 多次检验最终为合格品
            //若返修单返修完成，则会生成一个新的检验单并合格
            //若检验单为空，则生成新的单据并合格
            if (StringUtils.isEmpty(model.getPartInspection())) {

                partInspection = createPartInspection(model, MesExtEnumUtil.PART_INSPECTION_STATUS.QUALIFIED.getValue(), org);
                partInspectionRepository.save(partInspection);

                if (model.getSourceType() == MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.SINGLE.getValue()) {
                    MesProduceSn produceSn = checkProduceSn(model.getSn(), org);
                    produceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());
                    ConvertBean.serviceModelInitialize(produceSn, AuthUtil.getSessionUser().getUserName());
                    produceSnExtService.update(produceSn);
                }


            } else {

                //原单据不合格 现合格
                if (model.getPartInspection().getInspectionStatus() == MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue()) {
                    model.getPartInspection().setInspectionStatus(MesExtEnumUtil.PART_INSPECTION_STATUS.QUALIFIED.getValue());
                    ConvertBean.serviceModelUpdate(model.getPartInspection(), AuthUtil.getSessionUser().getUserName());
                    partInspectionRepository.save(model.getPartInspection());

                    //查询当前单据明细
                    List<MesPartInspectionDetail> resultDetailList = queryResultDetailList(model, org);

                    List<String> defectTypeNoExitList = new ArrayList<>();
                    for (MesPartInspectionDetail detail : model.getPartInspectionDetailList()) {

                        //根据页面上选择的位置 查询位置，若不存在则提示 位置不存在，请检查数据
                        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
                        DdlPreparedPack.getStringEqualPack(detail.getDefectLocation(), "defectTypeCode", packBean);
                        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.DEFECT_CATEGORY.POSITION.getValue(), "defectType", packBean);
                        MesDefectType defectType = defectTypeRepository.getByProperty(packBean);

                        if (StringUtils.isEmpty(defectType)) {
                            defectTypeNoExitList.add(detail.getDefectLocation());
                            continue;
                        }

                        detail.setDefectTypeId(defectType.getId());
                        detail.setId(null);
                        detail.setOrganizeCode(org);
                        detail.setPid(model.getPartInspection().getId());
                        ConvertBean.serviceModelInitialize(detail, AuthUtil.getSessionUser().getUserName());

                    }
                    if (!CollectionUtils.isEmpty(defectTypeNoExitList)) {
                        throw new ImppBusiException(String.format("【%s】位置不存在，请检查数据", defectTypeNoExitList));
                    }

                    partInspectionDetailRepository.saveAll(resultDetailList);
                    partInspectionDetailRepository.saveAll(model.getPartInspectionDetailList());

                    if (model.getSourceType() == MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.SINGLE.getValue()) {
                        MesProduceSn produceSn = checkProduceSn(model.getSn(), org);
                        produceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());
                        ConvertBean.serviceModelInitialize(produceSn, AuthUtil.getSessionUser().getUserName());
                        produceSnExtService.update(produceSn);
                    }
                    //移库
                    String srcLocate;
                    MesMove mesMove = getMesMove(model, org);
                    if(!Objects.isNull(mesMove)){
                        srcLocate = mesMove.getUmlgo();
                    }else{
                        srcLocate= configService.getCfgValue(org, "UMLGO");
                    }

                    //移库
                    MesMove move = createMove(model,srcLocate,configService.getCfgValue(org, "LGORT"), org);
                    moveRepository.save(move);
                }
            }

        } else {
            //零件检测详情不为空，则代表本次校验为不合格。

            //零件检测-单据为空，则是首次不合格
            MesPartInspection partInspection;
            if (StringUtils.isEmpty(model.getPartInspection())) {

                partInspection = createPartInspection(model, MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue(), org);
                model.setPartInspection(partInspectionRepository.save(partInspection));

                List<String> defectTypeNoExitList = new ArrayList<>();
                for (MesPartInspectionDetail detail : model.getPartInspectionDetailList()) {

                    //根据页面上选择的位置 查询位置，若不存在则提示 位置不存在，请检查数据
                    DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
                    DdlPreparedPack.getStringEqualPack(detail.getDefectLocation(), "defectTypeCode", packBean);
                    DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.DEFECT_CATEGORY.POSITION.getValue(), "defectType", packBean);
                    MesDefectType defectType = defectTypeRepository.getByProperty(packBean);

                    if (StringUtils.isEmpty(defectType)) {
                        defectTypeNoExitList.add(detail.getDefectLocation());
                        continue;
                    }

                    detail.setDefectTypeId(defectType.getId());
                    detail.setId(null);
                    detail.setOrganizeCode(org);
                    detail.setPid(partInspection.getId());
                    ConvertBean.serviceModelInitialize(detail, AuthUtil.getSessionUser().getUserName());

                }
                if (!CollectionUtils.isEmpty(defectTypeNoExitList)) {
                    throw new ImppBusiException(String.format("【%s】位置不存在，请检查数据", defectTypeNoExitList));
                }

                partInspectionDetailRepository.saveAll(model.getPartInspectionDetailList());

                //若是扫描条码 则判定条码可疑
                if (model.getSourceType() == MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.SINGLE.getValue()) {
                    MesProduceSn produceSn = checkProduceSn(model.getSn(), org);
                    produceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.SUSPICIOUS.getValue());
                    ConvertBean.serviceModelInitialize(produceSn, AuthUtil.getSessionUser().getUserName());
                    produceSnExtService.update(produceSn);
                }

                //移库
                MesMove move = createMove(model, configService.getCfgValue(org, "LGORT"), configService.getCfgValue(org, "UMLGO"), org);
                moveRepository.save(move);

            } else {

                model.getPartInspection().setInspectionStatus(MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue());
                ConvertBean.serviceModelUpdate(model.getPartInspection(), AuthUtil.getSessionUser().getUserName());
                partInspectionRepository.save(model.getPartInspection());

                //查询当前单据明细
                List<MesPartInspectionDetail> resultDetailList = queryResultDetailList(model, org);

                List<String> defectTypeNoExitList = new ArrayList<>();
                for (MesPartInspectionDetail detail : model.getPartInspectionDetailList()) {

                    //根据页面上选择的位置 查询位置，若不存在则提示 位置不存在，请检查数据
                    DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
                    DdlPreparedPack.getStringEqualPack(detail.getDefectLocation(), "defectTypeCode", packBean);
                    DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.DEFECT_CATEGORY.POSITION.getValue(), "defectType", packBean);
                    MesDefectType defectType = defectTypeRepository.getByProperty(packBean);

                    if (StringUtils.isEmpty(defectType)) {
                        defectTypeNoExitList.add(detail.getDefectLocation());
                        continue;
                    }

                    detail.setDefectTypeId(defectType.getId());
                    detail.setId(null);
                    detail.setOrganizeCode(org);
                    detail.setPid(model.getPartInspection().getId());
                    ConvertBean.serviceModelInitialize(detail, AuthUtil.getSessionUser().getUserName());

                }
                if (!CollectionUtils.isEmpty(defectTypeNoExitList)) {
                    throw new ImppBusiException(String.format("【%s】位置不存在，请检查数据", defectTypeNoExitList));
                }

                //若是扫描条码 则判定条码可疑
                if (model.getSourceType() == MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.SINGLE.getValue()) {
                    MesProduceSn produceSn = checkProduceSn(model.getSn(), org);
                    produceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.SUSPICIOUS.getValue());
                    ConvertBean.serviceModelInitialize(produceSn, AuthUtil.getSessionUser().getUserName());
                    produceSnExtService.update(produceSn);
                }

                partInspectionDetailRepository.saveAll(resultDetailList);
                partInspectionDetailRepository.saveAll(model.getPartInspectionDetailList());

                //移库
                MesMove move = createMove(model, configService.getCfgValue(org, "LGORT"), configService.getCfgValue(org, "UMLGO"), org);
                moveRepository.save(move);


            }

        }
    }

    private MesMove getMesMove(MesPartInspectionInputModel model, String org) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(model.getPartInspection().getId(),"partInspectionId",packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.MOVE_TYPE.RETURN_MOVE.getValue(),"moveType",packBean);
        MesMove mesMove = moveRepository.getByProperty(packBean);
        return mesMove;
    }

    /**
     * 获取NC-零件检测-单据
     *
     * @param inputModel
     * @param org
     * @return
     */
    private MesPartInspection getPartInspection(MesPartInspectionInputModel inputModel, String org) {

        //根据条码找最近一条单据
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(inputModel.getSn(), "sn", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        MesPartInspection partInspection = partInspectionRepository.getByProperty(packBean);

        //若单据经过了NC处理，则返回为空
        partInspection = checkPartInspection(partInspection, inputModel, org);

        return partInspection;
    }

    /**
     * 获取NC-零件检测-单据
     *
     * @param inputModel
     * @param org
     * @return
     */
    private MesPartInspection getPartInspectionByPartAndLot(MesPartInspectionInputModel inputModel, String org) {
        //根据扫描的条码查询条码是否存在
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(inputModel.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringEqualPack(inputModel.getLotNo(), "lotNo", packBean);
        DdlPreparedPack.getNumEqualPack(inputModel.getQty().intValue(), "qty", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.LOT.getValue(), "sourceType", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        MesPartInspection partInspection = partInspectionRepository.getByProperty(packBean);

        checkPartInspection(partInspection, inputModel, org);

        return partInspection;
    }

    /**
     * 查询当前单据明细
     *
     * @param model
     * @param org
     * @return
     */
    private List<MesPartInspectionDetail> queryResultDetailList(MesPartInspectionInputModel model, String org) {

        //根据扫描的条码查询条码是否存在
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(model.getPartInspection().getId(), "pid", packBean);
        List<MesPartInspectionDetail> resultDetailList = partInspectionDetailRepository.findByHqlWhere(packBean);
        resultDetailList.forEach(k -> {
            k.setIsDeleted(MesCommonConstant.TRUE_INTEGER);
            ConvertBean.serviceModelUpdate(model.getPartInspection(), AuthUtil.getSessionUser().getUserName());
        });

        return resultDetailList;
    }

    /**
     * 校验NC-零件检测-单据
     *
     * @param partInspection
     * @return
     */
    private MesPartInspection checkPartInspection(MesPartInspection partInspection, MesPartInspectionInputModel inputModel, String org) {

        //针对检测单-一次NC处理是一个单据 若单据经过了NC处理，则返回为空

        if (!StringUtils.isEmpty(partInspection)) {

            //可疑,待NC
            if (partInspection.getInspectionStatus() == MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue()
                    && partInspection.getNcStatus() == MesExtEnumUtil.PART_INSPECTION_NC_STATUS.CREATE.getValue()) {

                return partInspection;
            } else if (partInspection.getInspectionStatus() == MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue()
                    && partInspection.getNcStatus() == MesExtEnumUtil.PART_INSPECTION_NC_STATUS.SCRAP.getValue()) {

                //报废
                throw ImppExceptionBuilder.newInstance().setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail(String.format("【%s】条码已报废,请检查数据", partInspection.getSn()))
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .build();

            } else if (partInspection.getInspectionStatus() == MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue()
                    && partInspection.getNcStatus() == MesExtEnumUtil.PART_INSPECTION_NC_STATUS.REWORK.getValue()) {


                //已创建返修单
                DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
                DdlPreparedPack.getNumEqualPack(partInspection.getId(), "partInspectionId", packBean);
                DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
                MesReworkTask reworkTask = reworkTaskRepository.getByProperty(packBean);

                if (!StringUtils.isEmpty(reworkTask) && reworkTask.getStatus() == MesExtEnumUtil.REWORK_TASK_STATUS.CREATE.getValue()) {

                    //已创建返修单
                    throw ImppExceptionBuilder.newInstance().setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                            .setErrorDetail(String.format("【%s】已创建返修单,请检查数据", partInspection.getSn()))
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .build();
                } else if (!StringUtils.isEmpty(reworkTask) && reworkTask.getStatus() == MesExtEnumUtil.REWORK_TASK_STATUS.FINISH.getValue() &&
                        partInspection.getSourceType() == MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.SINGLE.getValue()) {
                    //返修单 进行返修
                    inputModel.setTransferFlg(true);
                }
                return partInspection;
            }
        }

        return null;
    }

    /**
     * 检查条码
     *
     * @param serialNumber
     * @param org
     * @return
     */

    private MesProduceSn checkProduceSn(String serialNumber, String org) {
        //根据扫描的条码查询条码是否存在
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(serialNumber, "productSn", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        MesProduceSn produceSn = produceSnRepository.getByProperty(packBean);

        //若不存在则提示条码不存在
        if (StringUtils.isEmpty(produceSn)) {
            throw new ImppBusiException(String.format("【%s】此条码不存在，请检查数据", serialNumber));
        }

        //根据条码状态判断 是不是已下线条码
        //若不是已下线的条码则提示 条码状态 请选择已下线的条码
        if (produceSn.getSnStatus() != MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue()) {
            throw new ImppBusiException(String.format("【%s】此条码状态为【%s】,请选择状态为【%s】状态的条码", serialNumber, MesExtEnumUtil.PRODUCE_SN_STATUS.valueOfDescription(produceSn.getSnStatus()),
                    MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getDescription()));
        }

//        //是否需要判断条码质量状态
//        if (produceSn.getQcStatus() != MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue()) {
//            throw new ImppBusiException(String.format("【%s】此条码质量状态为【%s】,请选择【%s】状态的条码", serialNumber, MesExtEnumUtil.PRODUCE_QC_STATUS.valueOfDescription(produceSn.getQcStatus()),
//                    MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getDescription()));
//        }
        return produceSn;
    }

    /**
     * 创建单据
     *
     * @param model
     * @param inspectionStatus
     * @param org
     * @return
     */
    private MesPartInspection createPartInspection(MesPartInspectionInputModel model, Integer inspectionStatus, String org) {
        MesPartInspection partInspection = new MesPartInspection();
        partInspection.setOrganizeCode(org);
        partInspection.setPartNo(model.getPart().getPartNo());
        partInspection.setPartName(model.getPart().getPartName());
        partInspection.setInspectionDate(TimeTool.getToday());
        partInspection.setInspectionStatus(inspectionStatus);
        partInspection.setNcStatus(MesExtEnumUtil.PART_INSPECTION_NC_STATUS.CREATE.getValue());
        partInspection.setSn(model.getSn());
        partInspection.setLotNo(model.getLotNo());
        partInspection.setQty(model.getQty().intValue());
        partInspection.setWorkCenterCode(model.getWorkCenterCode());
        partInspection.setShiftCode(model.getShiftCode());
        partInspection.setSourceType(model.getSourceType());
        ConvertBean.serviceModelInitialize(partInspection, AuthUtil.getSessionUser().getUserName());
        return partInspection;
    }

    /**
     * 移库
     *
     * @param model
     * @param source 来源
     * @param target 目标
     * @param org
     * @return
     */
    private MesMove createMove(MesPartInspectionInputModel model, String source, String target, String org) {
        GenSerialNoModel serialNoModel = new GenSerialNoModel("INPUT_DEFECT_ZRSUM");
        serialNoModel.setPartNo(model.getPart().getPartNo());
        ResultBean rb = syncFuncService.syncSerialNo(serialNoModel, AuthUtil.getSessionUser().getUserName(), org, 1);
        String zrsum = "";
        if (null != rb && !CollectionUtils.isEmpty(rb.getResultList())) {
            zrsum = (rb.getResultList().get(0)).toString();
        }

        MesMove move = new MesMove();
        move.setMatnr(model.getPart().getPartNo());
        move.setOrganizeCode(org);
        move.setFactoryCode(org);
        move.setLgort(source);
        move.setUmlgo(target);
        move.setMenge(model.getQty());
        move.setMeins(model.getPart().getUnit());
        move.setZrsum(zrsum);
        move.setPostDate(TimeTool.getToday());
        move.setPostTime(TimeTool.getTimeShortWithColon());
        move.setPartInspectionId(Objects.isNull(model.getPartInspection()) ? 0 : model.getPartInspection().getId());
        move.setMoveType(MesExtEnumUtil.MOVE_TYPE.SUSPICIOUS_MOVE.getValue());
        move.setProductSn(model.getSn());
        ConvertBean.serviceModelInitialize(move, AuthUtil.getSessionUser().getUserName());
        return move;
    }

    private MesPart checkPart(MesProduceSn produceSn, String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(produceSn.getPartNo(), "partNo", packBean);
        MesPart part = partRepository.getByProperty(packBean);

        //若不存在则提示物料信息不存在
        if (StringUtils.isEmpty(part)) {
            throw new ImppBusiException(String.format("【%s】此条码对应【%s】物料号不存在，请检查数据", produceSn.getProductSn(), produceSn.getPartNo()));
        }

        //根据条码查询零件类型（partType）
        if (StringUtils.isEmpty(part.getPartTypeCode())) {
            throw new ImppBusiException(String.format("【%s】此条码对应【%s】物料号未维护零件类型，请检查数据", produceSn.getProductSn(), produceSn.getPartNo()));
        }


        return part;
    }

    private MesPart checkPartByPartNo(String partNo, String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", packBean);
        MesPart part = partRepository.getByProperty(packBean);

        //若不存在则提示物料信息不存在
        if (StringUtils.isEmpty(part)) {
            throw new ImppBusiException(String.format("【%s】物料号不存在，请检查数据", partNo));
        }

        return part;
    }

    private MesPartTypePicture checkPartTypePicture(MesPart part, String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(part.getPartTypeCode(), "partTypeCode", packBean);
        MesPartTypePicture partTypePicture = partTypePictureRepository.getByProperty(packBean);

        //若不存在则提示零件类别不存在
        if (StringUtils.isEmpty(partTypePicture)) {
            throw new ImppBusiException(String.format("【%s】物料号对应的【%s】零件类别不存在，请检查数据", part.getPartNo(), part.getPartTypeCode()));
        }
        //若不存在则提示未维护正面图片
        if (StringUtils.isEmpty(partTypePicture.getFrontPictureName())) {
            throw new ImppBusiException(String.format("【%s】物料号对应的【%s】零件类别未维护正面图片，请检查数据", part.getPartNo(), part.getPartTypeCode()));
        }
        //若不存在则提示未维护反面图片
        if (StringUtils.isEmpty(partTypePicture.getBackPictureName())) {
            throw new ImppBusiException(String.format("【%s】物料号对应的【%s】零件类别未维护反面图片，请检查数据", part.getPartNo(), part.getPartTypeCode()));
        }

        return partTypePicture;
    }

    private List<MesLocationConfig> checkLocationConfig(String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue(), CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"rowLocation", "columnLocation"}, packBean);
        List<MesLocationConfig> configList = locationConfigRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(configList)) {
            throw new ImppBusiException(String.format("未配置位置，请检查数据"));
        }

        return configList;
    }

    private List<MesPartTypeDefect> checkPartTypeDefect(MesPart part, String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(part.getPartTypeCode(), "partTypeCode", packBean);
        List<MesPartTypeDefect> partTypeDefectList = partTypeDefectRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(partTypeDefectList)) {
            throw new ImppBusiException(String.format("未配置【%s】零件类型缺陷类型对应关系，请检查数据", part.getPartTypeCode()));
        }

        return partTypeDefectList;
    }

    private List<MesDefectWarnConfig> queryDefectAlarmConfigByLocationCodeList(MesPart part, Integer frontBack, List<String> locationCodeList, List<String> partTypeDefectCodeList,
                                                                               String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(part.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringEqualPack(part.getPartTypeCode(), "partTypeCode", packBean);
        DdlPreparedPack.getNumEqualPack(frontBack, "frontBack", packBean);
        DdlPreparedPack.getInPackList(locationCodeList, "defectLocation", packBean);
        DdlPreparedPack.getInPackList(partTypeDefectCodeList, "defectTypeCode", packBean);
        List<MesDefectWarnConfig> alarmConfigList = defectWarnConfigRepository.findByHqlWhere(packBean);

        return alarmConfigList;
    }

    private List<MesDefectWarnConfig> queryDefectAlarmConfig(MesPart part, Integer frontBack, String locationCode, List<String> partTypeDefectCodeList, String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(part.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringEqualPack(part.getPartTypeCode(), "partTypeCode", packBean);
        DdlPreparedPack.getNumEqualPack(frontBack, "frontBack", packBean);
        DdlPreparedPack.getStringEqualPack(locationCode, "defectLocation", packBean);
        DdlPreparedPack.getInPackList(partTypeDefectCodeList, "defectCode", packBean);
        List<MesDefectWarnConfig> alarmConfigList = defectWarnConfigRepository.findByHqlWhere(packBean);

        return alarmConfigList;
    }

    private MesPartInspectionViewModel getModel(MesPartInspection partInspection, MesPart part,MesPartTypePicture partTypePicture, MesPartInspectionInputModel inputModel ,
                                                Integer sourceType, String org) {

        Integer frontBack = inputModel.getFrontBack();
        List<MesPartInspectionDetail> detailList = new ArrayList<>();
        if (!StringUtils.isEmpty(partInspection) && partInspection.getInspectionStatus() != MesExtEnumUtil.PART_INSPECTION_STATUS.QUALIFIED.getValue()) {

            detailList = checkPartInspectionDetail(partInspection, org);
            if(!CollectionUtils.isEmpty(detailList)){
                //判断是否返工完成
                checkReworkFinished(org, detailList);
            }
        }
        //位置
        List<MesLocationConfig> locationConfigList = checkLocationConfig(org);
        List<String> locationCodeList = locationConfigList.stream().map(k -> k.getDefectLocation()).collect(Collectors.toList());

        //查询零件类型缺陷类型对应关系
        List<MesPartTypeDefect> partTypeDefectList = checkPartTypeDefect(part, org);
        List<String> partTypeDefectCodeList = partTypeDefectList.stream().map(k -> k.getDefectTypeCode()).collect(Collectors.toList());

        //查询预警数量
        List<MesDefectWarnConfig> alarmConfigList = queryDefectAlarmConfigByLocationCodeList(part, frontBack, locationCodeList, partTypeDefectCodeList, org);

        //根据不良类型+位置 若有数据，则表示需要标黄
        Map<String, List<MesDefectWarnConfig>> alarmConfigMap = alarmConfigList.stream().collect(Collectors.groupingBy(MesDefectWarnConfig::getDefectLocation));

        //返修完成需要标记绿色
        Map<String, List<MesPartInspectionDetail>> detailMap = detailList.stream().collect(Collectors.groupingBy(t->t.getDefectLocation()+t.getFrontBack()));

        locationConfigList.forEach(k -> {
            k.setFrontBack(frontBack);
            if (alarmConfigMap.containsKey(k.getDefectLocation())) {
                k.setIsflg(true);
                k.setColour(MesExtEnumUtil.COLOR.YELLOW.getValue());
            }

            if(detailMap.containsKey(k.getDefectLocation()+frontBack) &&
                    detailMap.get(k.getDefectLocation()+frontBack).stream().allMatch(MesPartInspectionDetail::getReworkFinished)){
                k.setIsflg(true);
                k.setColour(MesExtEnumUtil.COLOR.GREEN.getValue());
            }
        });

        MesPartInspectionViewModel model = new MesPartInspectionViewModel();
        model.setPart(part);
        if (!StringUtils.isEmpty(partInspection) && partInspection.getInspectionStatus() == MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue()
                && partInspection.getNcStatus() == MesExtEnumUtil.PART_INSPECTION_NC_STATUS.CREATE.getValue()) {
            model.setPartInspectionDetailList(detailList);
            model.setPartInspection(partInspection);
        }
        model.setPartTypePicture(partTypePicture);
        model.setLocationConfigList(locationConfigList);
        model.setMesDefectWarnConfigList(alarmConfigList);
        return model;

    }

    private void checkReworkFinished(String org, List<MesPartInspectionDetail> detailList) {
        List<Long> idList = detailList.stream().map(MesPartInspectionDetail::getId).collect(Collectors.toList());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getInPackList(idList, "partInspectionDetailId", packBean);
        List<MesReworkTaskDetail> mesReworkTaskDetailList = mesReworkTaskDetailRDao.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(mesReworkTaskDetailList)){
            Map<Long, List<MesReworkTaskDetail>> mesReworkTaskDetailMap = mesReworkTaskDetailList.stream().collect(Collectors.groupingBy(MesReworkTaskDetail::getPartInspectionDetailId));
            detailList.forEach(d->{
                if(mesReworkTaskDetailMap.containsKey(d.getId()) &&
                        mesReworkTaskDetailMap.get(d.getId()).stream().allMatch(t->MesExtEnumUtil.REWORK_TASK_STATUS.FINISH.getValue() == t.getStatus())){
                    d.setReworkFinished(true);
                }
            });
        }
    }

    private List<MesPartInspectionDetail> checkPartInspectionDetail(MesPartInspection partInspection, String org) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(partInspection.getId(), "pid", packBean);
        List<MesPartInspectionDetail> detailList = partInspectionDetailRepository.findByHqlWhere(packBean);

        return detailList;
    }

    @Override
    public void saveSusPartInspection(MesPartInspectionInputModel model, String org) {
        //
        MesPartInspection partInspection = new MesPartInspection();
        BeanUtils.copyProperties(model, partInspection);
        partInspection.setInspectionStatus(MesExtEnumUtil.PART_INSPECTION_STATUS.QUALIFIED.getValue());
        partInspection.setNcStatus(MesExtEnumUtil.PART_INSPECTION_NC_STATUS.CREATE.getValue());

        partInspectionRepository.save(partInspection);
        //
        String source = configService.getCfgValue(org, "LGORT");
        //移库
        String partNo = model.getPartNo();
        // 查询生产版本
        String workCenterCode = model.getWorkCenterCode();
        String productVersion = model.getProductVersion();

        if (!StringUtils.isEmpty(productVersion)) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
            DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(productVersion, "productVersion", ddlPackBean);

            MesProductVersion version = productVersionRepository.getByProperty(ddlPackBean);
            if (version != null) {
                source = version.getShipInventoryPoint();
            }
        }
        MesMove move = createMove(model, source, configService.getCfgValue(org, "UMLGO"), org);
        moveRepository.save(move);
    }
}
