package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesJisShippingService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesShippingLoadingCheckService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesSortShippingCheckService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.aspect.MonitorLog;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesSortShippingCheckModel;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesJisShipping;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingList;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingListDetail;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagement;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagementDetail;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-04-26 15:38
 * @Modify:
 **/
@Service
@Slf4j
public class MesSortShippingCheckService implements IMesSortShippingCheckService {

    @Autowired
    private MesShippingOrderManagementRepository shippingOrderManagementRepository;
    @Autowired
    private MesShippingOrderManagementDetailRepository shippingOrderManagementDetailRepository;
    @Autowired
    private MesProduceSnRepository produceSnRepository;

    @Autowired
    private IMesProduceSnExtService produceSnExtService;

    @Autowired
    private MesPartShippingGroupRepository partShippingGroupRepository;
    @Autowired
    private MesLoadingListRepository listRepository;

    @Autowired
    private MesLoadingListDetailRepository listDetailRepository;

    @Autowired
    private IMesShippingLoadingCheckService shippingLoadingCheckService;

    @Autowired
    private MesJisShippingRepository jisShippingRepository;

    @Autowired
    private IMesJisShippingService jisShippingService;
    @Autowired
    private MesLoadingListDetailRepository vehiclesOrderDetailRepository;
    @Autowired
    private MesLoadingListRepository vehiclesOrderRepository;

    @Autowired
    private MesCustomerPartRepository mesCustomerPartRepository;

    @Override
    public MesSortShippingCheckModel queryShippingOrderNo(MesShippingOrderManagement shippingOrderManagement) {


        //校验发运单是否存在
        DdlPackBean orderManagementPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(shippingOrderManagement.getShippingCode(), "shippingCode", orderManagementPackBean);
        MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(orderManagementPackBean);
        if (StringUtils.isEmpty(orderManagement)) {
            throw new ImppBusiException(String.format("【%s】此发运单不存在，请检查数据！", shippingOrderManagement.getShippingCode()));
        }

        //校验发运单状态 必须是未装车条码
        if (orderManagement.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getValue() && orderManagement.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
            throw new ImppBusiException(String.format("【%s】此发运单状态为【%s】,请选择状态为【%s】或【%s】状态的发运单", shippingOrderManagement.getShippingCode(),
                    MesExtEnumUtil.SHIPPING_ORDER_STATUS.valueOfDescription(orderManagement.getStatus()),
                    MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getDescription(), MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getDescription()));
        }

        //校验发运单扫描顺序
        List<Integer> statusForCheckList = new ArrayList<>();
        statusForCheckList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue());
        statusForCheckList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue());
        statusForCheckList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getValue());
        statusForCheckList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.PRINTED.getValue());
        orderManagementPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
        DdlPreparedPack.getNumberSmallerPack(orderManagement.getSeq(), "seq", orderManagementPackBean);
        DdlPreparedPack.getInPackList(statusForCheckList, "status", orderManagementPackBean);
        int queryCount = shippingOrderManagementRepository.findByHqlWhereCount(orderManagementPackBean);
        if (queryCount != 0) {
            throw new ImppBusiException(String.format("【%s】发运单对应排序【%s】前存在未扫描发运的发运单，请检查数据！",
                    orderManagement.getShippingCode(), orderManagement.getSeq()));
        }


        //根据扫描的发运单找到是否装车配置，若是，则根据发运单查询是否生成装车单，若没生成装车单则提示此发运单配置是否装车为是,但没生成装车单，请检查数据！
        if (orderManagement.getIsLoading() == MesCommonConstant.TRUE_INTEGER) {

            if (StringUtils.isEmpty(orderManagement.getOrderCode()) ) {
                throw new ImppBusiException(String.format("【%s】此发运单配置是否装车为是,但没绑定装车单号，请检查数据！", shippingOrderManagement.getShippingCode()));
            }


            DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(orderManagement.getOrderCode(), "orderCode", orderDetailPackBean);
            MesLoadingList loadingList = listRepository.getByProperty(orderDetailPackBean);
            if (StringUtils.isEmpty(loadingList)) {
                throw new ImppBusiException(String.format("【%s】此发运单配置是否装车为是,但没生成装车单，请检查数据！", shippingOrderManagement.getShippingCode()));
            }


            //判断是否先装车
            if (orderManagement.getIsFirstInstall() == MesCommonConstant.TRUE_INTEGER) {

                //若没进行装车校验，则提示【%s】此发运单对应的【%s】此装车单状态为【%s】，请选择状态为【%s】的数据
                if (loadingList.getStatus() != MesExtEnumUtil.MES_LOADING_STATUS.SCANNED.getValue()) {
                    throw new ImppBusiException(String.format("【%s】此发运单对应的【%s】此装车单状态为【%s】，请选择状态为【%s】的数据！",
                            shippingOrderManagement.getShippingCode(), loadingList.getOrderCode(),
                            MesExtEnumUtil.MES_LOADING_STATUS.valueOfDescription(loadingList.getStatus()),
                            MesExtEnumUtil.MES_LOADING_STATUS.SCANNED.getDescription()));
                }
            }

            //判断是否需要排序校验 默认排序则需要校验
            if (loadingList.getScanSeqWay() == MesExtEnumUtil.SHIPPING_GROUP_SCAN_CONFIRM_SEQ_MODE.DEFAULT_SORT.getValue()) {

                //获取此装车单中 状态为创建 已发布 发运中 seq最小的
                List<Integer> statusList = new ArrayList<>();
                statusList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue());
                statusList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue());
                statusList.add(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getValue());
                DdlPackBean sortOrderPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
                DdlPreparedPack.getNumEqualPack(loadingList.getId(), "loadingListId", sortOrderPackBean);
                DdlPreparedPack.getInPackList(statusList, "status", sortOrderPackBean);
                DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"seq"}, sortOrderPackBean);
                MesShippingOrderManagement sortTempOrder = shippingOrderManagementRepository.getByProperty(sortOrderPackBean);

                //若没有最小的，则此装车单 没有发运单扫描完成
                if (sortTempOrder.getId().longValue() != orderManagement.getId().longValue()) {
                    throw new ImppBusiException(String.format("【%s】此发运单扫描顺序错误，请扫描【%s】发运单！",
                            shippingOrderManagement.getShippingCode(), sortTempOrder.getOrderCode()));
                }


            }
        }

        MesSortShippingCheckModel model = new MesSortShippingCheckModel();
        if (!StringUtils.isEmpty(orderManagement.getCheckSeqCode())) {
            model.setIsScanLocationCode(true);
        }


        DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(orderManagement.getId(), "pid", orderDetailPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"custInfoSeq"}, orderDetailPackBean);
        List<MesShippingOrderManagementDetail> detailList = shippingOrderManagementDetailRepository.findByHqlWhere(orderDetailPackBean);

        detailList.forEach(k -> {
            if (!StringUtils.isEmpty(k.getBarcode())) {
                k.setIsScanFlg(MesCommonConstant.TRUE_INTEGER);
            }
        });

        Long count = detailList.stream().filter(k -> k.getIsScanFlg() == MesCommonConstant.TRUE_INTEGER).count();
        model.setIsScanCount(count.intValue());

        //查询最近扫描完成的装车单号
        DdlPackBean recentlyOrderPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue(), "status", recentlyOrderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"modifyDatetime"}, recentlyOrderPackBean);
        MesShippingOrderManagement recentlyVehiclesOrder = shippingOrderManagementRepository.getByProperty(recentlyOrderPackBean);

        model.setDetailList(detailList);
        model.setShippingCode(orderManagement.getShippingCode());
        if (!StringUtils.isEmpty(recentlyVehiclesOrder)) {

            model.setRecentlyScanShippingCode(recentlyVehiclesOrder.getShippingCode());
        }

        return model;
    }

    @Override
    public MesSortShippingCheckModel saveSn(MesSortShippingCheckModel model, String org) {

        //校验条码
        MesProduceSn produceSn = checkSn(model, org);

        //校验发运单是否存在
        MesShippingOrderManagement orderManagement = checkMesShippingOrderManagement(model, org);


        //根据条码找到对应的物料号，若物料号一致，则修改扫描flg，将flg改为true.
        boolean scanFlg = false;
        for (MesShippingOrderManagementDetail detail : model.getDetailList()) {
            if (detail.getPartNo().equals(produceSn.getPartNo()) && StringUtils.isEmpty(detail.getBarcode())) {


                //判断是否需要排序校验 默认排序则需要校验
                if (orderManagement.getScanSeqWay() == MesExtEnumUtil.SHIPPING_GROUP_SCAN_CONFIRM_SEQ_MODE.DEFAULT_SORT.getValue()) {
                    //获取明细中最小的
                    Optional<String> minPartNO = model.getDetailList().stream().filter(k -> StringUtils.isEmpty(k.getBarcode()))
                            .min(Comparator.comparingLong(k -> k.getCustInfoSeq())).map(k -> k.getPartNo());
                    //若扫描的不是最小顺序 则报错
                    if (minPartNO.isPresent() && !minPartNO.get().equals(produceSn.getPartNo())) {
                        throw new ImppBusiException(String.format("【%s】此条码扫描顺序错误，请检查数据！", model.getSn()));
                    }
                }

                scanFlg = true;
                detail.setBarcode(produceSn.getSerialNumber());

                if (StringUtils.isEmpty(orderManagement.getCheckSeqCode())) {
                    detail.setIsScanFlg(MesCommonConstant.TRUE_INTEGER);

                    //根据是否启动记忆判断 若启用记忆，则根据保存数据库
                    if (orderManagement.getIsEnableMemory() == MesCommonConstant.TRUE_INTEGER) {

                        //若发运单状态为创建或发布 则写入开始扫描时间
                        if (orderManagement.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() ||
                                orderManagement.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
                            orderManagement.setStartScanTime(TimeTool.getNowTime(true));
                        }

                        orderManagement.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getValue());
                        ConvertBean.serviceModelUpdate(orderManagement, AuthUtil.getSessionUser().getUserName());
                        update(orderManagement);

                        detail.setActualQty(1);
                        detail.setStatus(MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
                        ConvertBean.serviceModelUpdate(detail, AuthUtil.getSessionUser().getUserName());
                        updateDetail(detail);
                        //内部条码更新条码状态
                        if (!Objects.isNull(produceSn.getId())) {
                            produceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.SHIPPED.getValue());
                            ConvertBean.serviceModelUpdate(detail, AuthUtil.getSessionUser().getUserName());
                            produceSnExtService.update(produceSn);
                        }
                    }
                }
                break;
            }
        }

        if (!scanFlg) {
            throw new ImppBusiException(String.format("【%s】此条码对应的【%s】物料号与发运单中物料号不匹配，请检查数据！", model.getSn(), produceSn.getPartNo()));
        }

        Long count = model.getDetailList().stream().filter(k -> k.getIsScanFlg() == MesCommonConstant.TRUE_INTEGER).count();
        model.setIsScanCount(count.intValue());

        //当前发运单全部扫描完成 并且不需要扫位置码
        if (count.intValue() == model.getDetailList().size() && StringUtils.isEmpty(orderManagement.getCheckSeqCode())) {

            //保存数据库
            saveDate(orderManagement, model, org);
        }

        return model;
    }

    @Override
    public MesSortShippingCheckModel saveSnAndLocationCode(MesSortShippingCheckModel model, String org) {

        //校验条码
        MesProduceSn produceSn = checkMesProduceSn(model, org);


        //校验发运单是否存在
        MesShippingOrderManagement orderManagement = checkMesShippingOrderManagement(model, org);

        //根据条码找到对应的物料号，若物料号一致，则修改扫描flg，将flg改为true.
        boolean scanFlg = false;
        for (MesShippingOrderManagementDetail detail : model.getDetailList()) {
            if (detail.getPartNo().equals(produceSn.getPartNo())) {
                scanFlg = true;
                detail.setIsScanFlg(MesCommonConstant.TRUE_INTEGER);
                detail.setBarcode(produceSn.getSerialNumber());
                break;
            }
        }

        if (!scanFlg) {
            throw new ImppBusiException(String.format("【%s】此条码对应的【%s】物料号与发运单中物料号不匹配，请检查数据！", model.getSn(), produceSn.getPartNo()));
        }

        Long count = model.getDetailList().stream().filter(k -> k.getIsScanFlg() == MesCommonConstant.TRUE_INTEGER).count();

        //当前发运单全部扫描完成
        if (count.intValue() == model.getDetailList().size()) {

            //保存数据库
            saveDate(orderManagement, model, org);

        }

        return model;
    }

    @Override
    public MesShippingOrderManagement queryShippingOrderNoByAutoPushOrder(String shippingGroupNo, String org) {

        //查询最近扫描完成的装车单号
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue(), "status", orderPackBean);
//        DdlPreparedPack.getStringEqualPack(shippingGroupNo, "shippingGroupCode", orderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"seq"}, orderPackBean);
        MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(orderPackBean);

        return orderManagement;

    }

    private MesProduceSn checkSn(MesSortShippingCheckModel model, String org) {
        //校验条码是否重复扫描
        Optional<MesShippingOrderManagementDetail> optional = model.getDetailList().stream().filter(k -> !StringUtils.isEmpty(k.getBarcode()) && k.getBarcode().equals(model.getSn())).findFirst();
        if (optional.isPresent()) {
            throw new ImppBusiException(String.format("【%s】此条码已经扫描过，请检查数据！", model.getSn()));
        }
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getSn(), "productSn", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        MesProduceSn produceSn = produceSnRepository.getByProperty(packBean);
        if (!StringUtils.isEmpty(produceSn)) {
            //校验状态
            checkStatus(model, produceSn);
        }else{
            //外部条码解析
            MesShippingOrderManagementDetail mesShippingOrderManagementDetail = getMesShippingOrderManagementDetail(model, org);
            //封装信息
            produceSn = new MesProduceSn();
            produceSn.setSerialNumber(model.getSn());
            produceSn.setPartNo(mesShippingOrderManagementDetail.getPartNo());
        }

        return produceSn;

    }

    private MesShippingOrderManagementDetail getMesShippingOrderManagementDetail(MesSortShippingCheckModel model, String org) {
        List<MesShippingOrderManagementDetail> detailList = model.getDetailList().stream().filter(k -> !StringUtils.isEmpty(k.getCustPartNo()) && model.getSn().startsWith(k.getCustPartNo()) && !StringUtils.isEmpty(k.getBarcode())).collect(Collectors.toList());
        MesShippingOrderManagementDetail mesShippingOrderManagementDetail = null;
        for (MesShippingOrderManagementDetail managementDetail : detailList) {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
            DdlPreparedPack.getStringEqualPack(managementDetail.getPartNo(), "erpPartNo", packBean);
            DdlPreparedPack.getStringEqualPack(managementDetail.getCustPartNo(), "custPartNo", packBean);
            if(mesCustomerPartRepository.isExitByHql(packBean)){
                mesShippingOrderManagementDetail = managementDetail;
                break;
            }
        }
        if(Objects.isNull(mesShippingOrderManagementDetail)){
            throw new ImppBusiException(String.format("【%s】此条码未匹配到数据，请检查数据！", model.getSn()));
        }
        return mesShippingOrderManagementDetail;
    }


    private MesProduceSn checkMesProduceSn(MesSortShippingCheckModel model, String org) {
        //校验条码是否已经扫描
        Optional<MesShippingOrderManagementDetail> optional = model.getDetailList().stream().filter(k -> !StringUtils.isEmpty(k.getBarcode()) && k.getBarcode().equals(model.getSn())).findFirst();
        if (optional.isPresent()) {
            throw new ImppBusiException(String.format("【%s】此条码已经扫描过，请检查数据！", model.getSn()));
        }
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getSn(), "productSn", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        MesProduceSn produceSn = produceSnRepository.getByProperty(packBean);


        if (StringUtils.isEmpty(produceSn)) {
            throw new ImppBusiException(String.format("【%s】此产品条码不存在，请检查数据！", model.getSn()));
        }
        //校验状态
        checkStatus(model, produceSn);
        return produceSn;
    }

    private void checkStatus(MesSortShippingCheckModel model, MesProduceSn produceSn) {
        //校验产品条码状态 必须是状态为已下线条码
        if (produceSn.getSnStatus() != MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue()) {
            throw new ImppBusiException(String.format("【%s】此条码状态为【%s】,请选择状态为【%s】状态的条码", model.getSn(), MesExtEnumUtil.PRODUCE_SN_STATUS.valueOfDescription(produceSn.getSnStatus()),
                    MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getDescription()));
        }
        //校验产品质量状态 必须是状态为合格条码
        if (produceSn.getQcStatus() != MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue()) {
            throw new ImppBusiException(String.format("【%s】此条码质量状态为【%s】,请选择【%s】状态的条码", model.getSn(), MesExtEnumUtil.PRODUCE_QC_STATUS.valueOfDescription(produceSn.getQcStatus()),
                    MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getDescription()));
        }
    }

    private MesShippingOrderManagement checkMesShippingOrderManagement(MesSortShippingCheckModel model, String org) {
        //校验发运单是否存在
        DdlPackBean orderManagementPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getShippingCode(), "shippingCode", orderManagementPackBean);
        MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(orderManagementPackBean);
        if (StringUtils.isEmpty(orderManagement)) {
            throw new ImppBusiException(String.format("【%s】此发运单不存在，请检查数据！", model.getShippingCode()));
        }
        return orderManagement;
    }

    public void saveDate(MesShippingOrderManagement orderManagement, MesSortShippingCheckModel model, String org) {
        String userName = AuthUtil.getSessionUser().getUserName();
        //若发运单状态为创建或发布 则写入开始扫描时间
        if (orderManagement.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() ||
                orderManagement.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
            orderManagement.setStartScanTime(TimeTool.getNowTime(true));
        }

        //发运单状态修改为校验完成
        orderManagement.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue());
        orderManagement.setEndScanTime(TimeTool.getNowTime(true));

        //发运单明细状态修改为校验完成
        DdlPackBean orderManagementDetailPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(orderManagement.getId(), "pid", orderManagementDetailPackBean);
        List<MesShippingOrderManagementDetail> orderManagementDetailList = shippingOrderManagementDetailRepository.findByHqlWhere(orderManagementDetailPackBean);
        List<MesShippingOrderManagementDetail> detailList = model.getDetailList();
        Map<Long, List<MesShippingOrderManagementDetail>> custInfoSeqMap = detailList.stream().collect(Collectors.groupingBy(MesShippingOrderManagementDetail::getCustInfoSeq));

        List<MesProduceSn> produceSnList = new ArrayList<>();
        for (MesShippingOrderManagementDetail detail : orderManagementDetailList) {
            //根据扫描的条码查询条码是否存在
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
            if (orderManagement.getIsEnableMemory() == MesCommonConstant.TRUE_INTEGER) {
                detail.setBarcode(StringUtil.isEmpty(detail.getBarcode()) ? model.getSn() : detail.getBarcode());
                DdlPreparedPack.getStringEqualPack(StringUtil.isEmpty(detail.getBarcode()) ? model.getSn() : detail.getBarcode(), "productSn", packBean);
            } else {
                List<MesShippingOrderManagementDetail> mesShippingOrderManagementDetails = custInfoSeqMap.get(detail.getCustInfoSeq());
                MesShippingOrderManagementDetail managementDetail = mesShippingOrderManagementDetails.get(0);

                detail.setBarcode(StringUtil.isEmpty(managementDetail.getBarcode()) ? model.getSn() : detail.getBarcode());
                DdlPreparedPack.getStringEqualPack(StringUtil.isEmpty(managementDetail.getBarcode()) ? model.getSn() : detail.getBarcode(), "productSn", packBean);
            }
            detail.setActualQty(1);
            detail.setStatus(MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
            ConvertBean.serviceModelUpdate(detail, userName);

            DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
            MesProduceSn produceSn = produceSnRepository.getByProperty(packBean);
            if(!Objects.isNull(produceSn)){
                produceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.SHIPPED.getValue());
                ConvertBean.serviceModelUpdate(produceSn, userName);
                produceSnList.add(produceSn);
            }

        }

        if(!CollectionUtils.isEmpty(produceSnList)){
            produceSnRepository.saveAll(produceSnList);
        }
        update(orderManagement);
        shippingOrderManagementDetailRepository.saveAll(orderManagementDetailList);
        //1.更新装车单明细对应的发运单状态=发运
        DdlPackBean vehiclesOrderDetailPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(orderManagement.getShippingCode(), "shippingCode", vehiclesOrderDetailPackBean);
        MesLoadingListDetail vehiclesOrderDetail = vehiclesOrderDetailRepository.getByProperty(vehiclesOrderDetailPackBean);
        if(!StringUtil.isEmpty(vehiclesOrderDetail)){
            vehiclesOrderDetail.setStatus(MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
            ConvertBean.serviceModelUpdate(vehiclesOrderDetail, userName);
            vehiclesOrderDetailRepository.save(vehiclesOrderDetail);
        }

        DdlPackBean orderManagementPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(orderManagement.getOrderCode(), "orderCode", orderManagementPackBean);
        DdlPreparedPack.getNumNOEqualPack(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue(), "status", orderManagementPackBean);
        boolean exitByHql = shippingOrderManagementRepository.isExitByHql(orderManagementPackBean);
        //2.装车单中所有发运单全部变成发运回传
        if(!exitByHql){
            List<MesJisShipping> jisShippingList=new ArrayList<>();
            DdlPackBean jisBean = DdlPackBean.getDdlPackBean(org);
            DdlPreparedPack.getStringEqualPack(orderManagement.getOrderCode(), "orderCode", jisBean);
            List<MesShippingOrderManagement> shippingOrderManagementList = shippingOrderManagementRepository.findByHqlWhere(jisBean);
            for (MesShippingOrderManagement mesShippingOrderManagement : shippingOrderManagementList) {
                DdlPackBean detailPackBean = DdlPackBean.getDdlPackBean(org);
                DdlPreparedPack.getNumEqualPack(mesShippingOrderManagement.getId(), "pid", detailPackBean);
                List<MesShippingOrderManagementDetail> shippingOrderManagementDetails = shippingOrderManagementDetailRepository.findByHqlWhere(detailPackBean);
                //当前装车单所有发运单明细 需要报工的数据
                jisShippingList = jisShippingService.getJisShipping(mesShippingOrderManagement, shippingOrderManagementDetails, org);
            }
            //保存当前发运单明细报工
            jisShippingRepository.saveAll(jisShippingList);

            if(!StringUtil.isEmpty(vehiclesOrderDetail)){
                MesLoadingList loadingList = vehiclesOrderRepository.getById(vehiclesOrderDetail.getPid());
                if (!StringUtils.isEmpty(loadingList)) {
                    loadingList.setStatus(MesExtEnumUtil.MES_LOADING_STATUS.ASN.getValue());
                    ConvertBean.serviceModelUpdate(loadingList, userName);
                    shippingLoadingCheckService.update(loadingList);
                }
            }


        }

    }
    @Override
    @MonitorLog
    public void update(MesShippingOrderManagement item) {
        shippingOrderManagementRepository.update(item);
    }

    @Override
    @MonitorLog
    public void updateDetail(MesShippingOrderManagementDetail item) {
        shippingOrderManagementDetailRepository.update(item);
    }
}
