package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.print.strategy;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.print.IPrintTemplateStrategyService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintDataModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.api.iservice.base.IConfigService;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerPartRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.hutool.core.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.Objects;

/**
 * @Description : 广州打印主条码使用
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2024/6/6 17:03
 * @Modify:
 **/
@Component
@Slf4j
public class AionPrintStrategy implements IPrintTemplateStrategyService {
    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;

    @Autowired
    private IMesPartService mesPartService;

    @Autowired
    private IConfigService configService;

    @Autowired
    private MesCustomerPartRepository mesCustomerPartRDao;


    @Override
    public MesProduceSnPrintModel execute(GenSerialNoModel genSerialNoModel, MesProduceSnPrintModel mesProduceSnPrintModel, MesNumberRule numberRule , StepResult stepResult, StationRequestBean reqBean, Boolean isStep) {
        // 获取工厂名称
        String organizeCode = mesProduceSnPrintModel.getOrganizeCode();
        MesConfig mesConfig = configService.getMesConfigByCfgCode(MesPcnExtConstWords.ORGANIZE_NAME, organizeCode);
        if (null == mesConfig || StringUtils.isEmpty(mesConfig.getCfgValue())) {
            throw ImppExceptionBuilder.newInstance().setSystemID(CommonEnumUtil.SOFT_TYPE.MES_PCN.getCode()).setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode()).setErrorDetail("没有维护当前工厂的系统参数!").build();
        }
        //获取工厂名称
        String organizeName = mesConfig.getCfgValue();
        String format ="yyyy/MM/dd";
        MesCustomerPart mesCustomerPart = getMesCustomerPart(mesProduceSnPrintModel);
        //物料信息
        MesPart mesPart = mesPartService.getMesPartByPartNo(mesProduceSnPrintModel.getPartNo(), organizeCode);
        if (!isStep){
            for (int i = 0; i < mesProduceSnPrintModel.getPrintQty(); i++) {
                ResultBean resultBean = syncFuncService.syncSerialNo(genSerialNoModel, mesProduceSnPrintModel.getUserName(), organizeCode, 1);
                String sn = resultBean.getResultList().get(0).toString();
                int versionNoLength = numberRule.getSpiltRule().substring(numberRule.getSpiltRule().lastIndexOf(",") + 1).length();
                Integer serialNoLength = numberRule.getSerialnoLength();
                String snPrefix = sn.substring(0, sn.length() - versionNoLength - serialNoLength);
                String snSuffix = sn.substring(sn.length() - versionNoLength);
                String serialNo = sn.substring(sn.length() - versionNoLength - serialNoLength, sn.length() - versionNoLength);
                String serialNo34 = toBase34(Integer.parseInt(serialNo));
                serialNo34 = stringFormat(serialNo34, 4, false, "0");
                sn = snPrefix + serialNo34 + snSuffix;
                //保存条码信息
                MesProduceSn produceSn = generateMesProduceSn(mesPart, sn, mesProduceSnPrintModel.getUserName(), mesProduceSnPrintModel.getQty(), format);
                //封装打印信息
                MesProduceSnPrintDataModel printDataModel = getModel(produceSn, serialNo, mesCustomerPart.getCustPartNo(), organizeName);
                mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(printDataModel);
                mesProduceSnPrintModel.getMesProduceSnList().add(produceSn);
                //保存打印记录
                MesPrintedSnLog snLog = new MesPrintedSnLog();
                ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
                BeanUtil.copyProperties(printDataModel,snLog);
                mesProduceSnPrintModel.getMesPrintedSnLogList().add(snLog);
            }
        }else {
                String sn = mesProduceSnPrintModel.getProductSn();
                MesProduceSn mesProduceSn = mesProduceSnPrintModel.getMesProduceSnList().get(0);
                int versionNoLength = numberRule.getSpiltRule().substring(numberRule.getSpiltRule().lastIndexOf(",") + 1).length();
                Integer serialNoLength = numberRule.getSerialnoLength();
                String snPrefix = sn.substring(0, sn.length() - versionNoLength - serialNoLength);
                String snSuffix = sn.substring(sn.length() - versionNoLength);
                String serialNo = sn.substring(sn.length() - versionNoLength - serialNoLength, sn.length() - versionNoLength);
                String serialNo34 = toBase34(Integer.parseInt(serialNo));
                serialNo34 = stringFormat(serialNo34, 4, false, "0");
                sn = snPrefix + serialNo34 + snSuffix;
                //封装打印信息
                MesProduceSnPrintDataModel printDataModel = getModel(mesProduceSn, serialNo, mesCustomerPart.getCustPartNo(), organizeName);
                printDataModel.setBarcode(sn);
                mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(printDataModel);
                //保存打印记录
                MesPrintedSnLog snLog = new MesPrintedSnLog();
                ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
                BeanUtil.copyProperties(printDataModel,snLog);
                mesProduceSnPrintModel.getMesPrintedSnLogList().add(snLog);
        }
        return mesProduceSnPrintModel;
    }

    private MesProduceSnPrintDataModel getModel(MesProduceSn produceSn, String no, String custPartNo, String factoryName) {
        MesProduceSnPrintDataModel mesProduceSnPrintDataModel = new MesProduceSnPrintDataModel();
        mesProduceSnPrintDataModel.setFactoryName(factoryName);
        mesProduceSnPrintDataModel.setPartNo(produceSn.getPartNo());
        mesProduceSnPrintDataModel.setPartName(produceSn.getPartName());
        mesProduceSnPrintDataModel.setNo(no);
        mesProduceSnPrintDataModel.setBarcode(produceSn.getProductSn());
        mesProduceSnPrintDataModel.setCustPartNo(custPartNo);
        mesProduceSnPrintDataModel.setProductDate(produceSn.getLotNo());
        return mesProduceSnPrintDataModel;
    }

    // 34进制字符集
    private String toBase34(int number) {
        String base34Chars = "0123456789ABCDEFGHJKLMNPQRSTUVWXYZ";
        StringBuilder sb = new StringBuilder();

        while (number > 0) {
            int remainder = number % 34;
            sb.append(base34Chars.charAt(remainder));
            number /= 34;
        }
        return sb.reverse().toString();
    }


    private String stringFormat(String data, Integer length) {
        return stringFormat(data, length, true, "*");
    }

    private String stringFormat(String data, Integer length, boolean left, String symbol) {
        //数据长途大于填充位数直接返回
        if (data.length() >= length) {
            return data;
        }
        StringBuilder dataBuffer = new StringBuilder();
        //右拼接
        if (left) {
            dataBuffer.append(data);
            for (int i = 0; i < length - data.length(); i++) {
                dataBuffer.append(symbol);
            }
        }

        //左拼接
        if (!left) {
            for (int i = 0; i < length - data.length(); i++) {
                dataBuffer.append(symbol);
            }
            dataBuffer.append(data);
        }
        return dataBuffer.toString();
    }

    private MesProduceSn generateMesProduceSn(MesPart mesPart, String sn, String userName, Double qty, String format) {
        MesProduceSn mesProduceSn = new MesProduceSn();
        mesProduceSn.setSerialNumber(snowflakeIdMaker.nextId() + "");
        mesProduceSn.setProductSn(sn);
        mesProduceSn.setCustSn(sn);
        mesProduceSn.setPartNo(mesPart.getPartNo());
        mesProduceSn.setPartName(mesPart.getPartName());
        mesProduceSn.setProcessLabelTemplate(mesPart.getProcessLabelTemplate());
        mesProduceSn.setCustLabelTemplate(mesPart.getCustLabelTemplate());
        mesProduceSn.setProdLabelTemplate(mesPart.getProductLabelTemplate());
        mesProduceSn.setQty(qty);
        mesProduceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.CREATE.getValue());
        mesProduceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());
        mesProduceSn.setLotNo(TimeTool.getNowTime(format));
        mesProduceSn.setPrintCount(MesPcnExtConstWords.ONE);
        mesProduceSn.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.PRINTED.getValue());
        mesProduceSn.setOrganizeCode(mesPart.getOrganizeCode());
        ConvertBean.serviceModelInitialize(mesProduceSn, userName);
        return mesProduceSn;
    }

    private MesCustomerPart getMesCustomerPart(MesProduceSnPrintModel mesProduceSnPrintModel) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getPartNo(), "erpPartNo", ddlPackBean);
        MesCustomerPart mesCustomerPart = mesCustomerPartRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesCustomerPart)) {
            MesPcnException.throwMesBusiException("物料【%s】客户零件关系未维护", mesProduceSnPrintModel.getPartNo());
        }
        return mesCustomerPart;
    }
}
