package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.print.strategy;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.print.IPrintTemplateStrategyService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintDataModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.api.iservice.base.IConfigService;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.pcn.util.DateUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerPartRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.hutool.core.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.Objects;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2024/6/17 16:10
 * @Modify:
 **/
@Component
@Slf4j
public class GqxnyPrintStrategy implements IPrintTemplateStrategyService {
    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;

    @Autowired
    private IMesPartService mesPartService;

    @Autowired
    private IConfigService configService;

    @Autowired
    private MesCustomerPartRepository mesCustomerPartRDao;

    @Override
    public MesProduceSnPrintModel execute(GenSerialNoModel genSerialNoModel, MesProduceSnPrintModel mesProduceSnPrintModel, MesNumberRule numberRule,StepResult stepResult, StationRequestBean reqBean,Boolean isStep) {
        //获取客户零件号
        String custPartNo = getMesCustomerPart(mesProduceSnPrintModel).getCustPartNo();
        String format = DateUtil.BASE_FORMAT;
        // 获取工厂名称
        String organizeCode = mesProduceSnPrintModel.getOrganizeCode();
        MesConfig mesConfig = configService.getMesConfigByCfgCode(MesPcnExtConstWords.ORGANIZE_NAME, organizeCode);
        if (null == mesConfig || StringUtils.isEmpty(mesConfig.getCfgValue())) {
            throw ImppExceptionBuilder.newInstance().setSystemID(CommonEnumUtil.SOFT_TYPE.MES_PCN.getCode()).setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode()).setErrorDetail("没有维护当前工厂的系统参数!").build();
        }
        //获取工厂名称
        String organizeName = mesConfig.getCfgValue();
        if (!isStep){
            //物料信息
            MesPart mesPart = mesPartService.getMesPartByPartNo(mesProduceSnPrintModel.getPartNo(), organizeCode);
            Date date = new Date();
            //----广乘 45位 GQXNY
            //{custPartNo}{SPILTURE}{custCode}{spiltrure}{spiltrule}{year}{serialNo}{spilture}{spiltrule}
            //客户零件号-扩展位-客户代码-预留位000-产线1-年月日-条码-扩展0000-追溯位+
            //零件长号---17位,不足17位 右边补*
            genSerialNoModel.setRuleCode(numberRule.getRuleCode());
            String custPartNoFormat = stringFormat(custPartNo, 17);
            genSerialNoModel.setCustPartNo(custPartNoFormat);
            //拓展位 2位 00 包含0~9,A~Z 不使用时，补“0”填充 {SPILTRULE}
            //供应商代码 7位 供应商代码 0~9,A~Z;不足7位 右测补*
            MesConfig supplierConfig = configService.getMesConfigByCfgCodeKeyOrg(MesPcnExtConstWords.SUPPLIER_CODE, MesExtEnumUtil.MES_PRINT_TEMPLATE.GQXNY.getValue(), organizeCode);
            String supplierCode = supplierConfig.getCfgValue();
            String supplierCodeFormat = stringFormat(supplierCode, 7);
            genSerialNoModel.setCustCode(supplierCodeFormat);
            //扩展位 3位 预留扩展位0~9 A~Z 不使用时，补 0 {SPILTRULE}
            //产线 1位 供应商产线号，表示由数字1~9 和字母A~Z(I/O除外)组成; {SPILTRULE}
            //年月日 6位 yyMMdd 零部件生产日期，年月日各2位
            String year = TimeTool.pareDateToString("yyMMdd", date);
            genSerialNoModel.setYear(year);
            //流水号 4位 采用34进制 0001~ZZZZ (I、O除外)顺位进行标注 (同广汽 & 广新) {serialNo}
            //扩展位 4位 预留扩展位信息，包含0~9和字母A~Z,不使用时，补“0” 填充;  {SPILTRULE}
            //追溯标识 1位 + 代表精确按件追溯，$标识按批追溯, % 代表按生产方式追溯  {SPILTRULE}
            for (int i = 0; i < mesProduceSnPrintModel.getPrintQty(); i++) {
                ResultBean resultBean = syncFuncService.syncSerialNo(genSerialNoModel, mesProduceSnPrintModel.getUserName(), organizeCode, 1);
                String sn = resultBean.getResultList().get(0).toString();
                String snPrefix = sn.substring(0, sn.length() - 12);
                String snSuffix = sn.substring(sn.length() - 5);
                String serialNo = sn.substring(sn.length() - 12, sn.length() - 5);
                String serialNo34 = toBase34(Integer.parseInt(serialNo));
                serialNo34 = stringFormat(serialNo34, 4, false, "0");
                sn = snPrefix + serialNo34 + snSuffix;
                //保存条码信息
                MesProduceSn produceSn = generateMesProduceSn(mesPart, sn, mesProduceSnPrintModel.getUserName(), mesProduceSnPrintModel.getQty(), format);
                //封装打印信息
                MesProduceSnPrintDataModel printDataModel = getModel(produceSn, serialNo, custPartNo, organizeName);
                printDataModel.setSupplierCode(supplierCode);
                mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(printDataModel);

                //保存打印记录
                MesPrintedSnLog snLog = new MesPrintedSnLog();
                ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
                BeanUtil.copyProperties(printDataModel, snLog);
                mesProduceSnPrintModel.getMesProduceSnList().add(produceSn);
            }
        }else {
            String sn = mesProduceSnPrintModel.getProductSn();
            MesProduceSn mesProduceSn = mesProduceSnPrintModel.getMesProduceSnList().get(0);
            int versionNoLength = numberRule.getSpiltRule().substring(numberRule.getSpiltRule().lastIndexOf(",") + 1).length();
            Integer serialNoLength = numberRule.getSerialnoLength();
            String snPrefix = sn.substring(0, sn.length() - versionNoLength - serialNoLength);
            String snSuffix = sn.substring(sn.length() - versionNoLength);
            String serialNo = sn.substring(sn.length() - versionNoLength - serialNoLength, sn.length() - versionNoLength);
            String serialNo34 = toBase34(Integer.parseInt(serialNo));
            serialNo34 = stringFormat(serialNo34, 4, false, "0");
            sn = snPrefix + serialNo34 + snSuffix;
            //封装打印信息
            MesProduceSnPrintDataModel printDataModel = getModel(mesProduceSn, serialNo, custPartNo, organizeName);
            printDataModel.setBarcode(sn);
            mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(printDataModel);
            //保存打印记录
            MesPrintedSnLog snLog = new MesPrintedSnLog();
            ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
            BeanUtil.copyProperties(printDataModel,snLog);
            mesProduceSnPrintModel.getMesPrintedSnLogList().add(snLog);
        }
        return mesProduceSnPrintModel;
    }

    private MesCustomerPart getMesCustomerPart(MesProduceSnPrintModel mesProduceSnPrintModel) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getPartNo(), "erpPartNo", ddlPackBean);
        MesCustomerPart mesCustomerPart = mesCustomerPartRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesCustomerPart)) {
            MesPcnException.throwMesBusiException("物料【%s】客户零件关系未维护", mesProduceSnPrintModel.getPartNo());
        }
        return mesCustomerPart;
    }

    private String stringFormat(String data, Integer length) {
        return stringFormat(data, length, true, "*");
    }

    private String stringFormat(String data, Integer length, boolean left, String symbol) {
        //数据长途大于填充位数直接返回
        if (data.length() >= length) {
            return data;
        }
        StringBuilder dataBuffer = new StringBuilder();
        //右拼接
        if (left) {
            dataBuffer.append(data);
            for (int i = 0; i < length - data.length(); i++) {
                dataBuffer.append(symbol);
            }
        }

        //左拼接
        if (!left) {
            for (int i = 0; i < length - data.length(); i++) {
                dataBuffer.append(symbol);
            }
            dataBuffer.append(data);
        }
        return dataBuffer.toString();
    }

    // 34进制字符集
    private String toBase34(int number) {
        String base34Chars = "0123456789ABCDEFGHJKLMNPQRSTUVWXYZ";
        StringBuilder sb = new StringBuilder();

        while (number > 0) {
            int remainder = number % 34;
            sb.append(base34Chars.charAt(remainder));
            number /= 34;
        }
        return sb.reverse().toString();
    }

    private MesProduceSn generateMesProduceSn(MesPart mesPart, String sn, String userName, Double qty, String format) {
        MesProduceSn mesProduceSn = new MesProduceSn();
        mesProduceSn.setSerialNumber(snowflakeIdMaker.nextId() + "");
        mesProduceSn.setProductSn(sn);
        mesProduceSn.setCustSn(sn);
        mesProduceSn.setPartNo(mesPart.getPartNo());
        mesProduceSn.setPartName(mesPart.getPartName());
        mesProduceSn.setProcessLabelTemplate(mesPart.getProcessLabelTemplate());
        mesProduceSn.setCustLabelTemplate(mesPart.getCustLabelTemplate());
        mesProduceSn.setProdLabelTemplate(mesPart.getProductLabelTemplate());
        mesProduceSn.setQty(qty);
        mesProduceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.CREATE.getValue());
        mesProduceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());
        mesProduceSn.setLotNo(TimeTool.getNowTime(format));
        mesProduceSn.setPrintCount(MesPcnExtConstWords.ONE);
        mesProduceSn.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.PRINTED.getValue());
        mesProduceSn.setOrganizeCode(mesPart.getOrganizeCode());
        ConvertBean.serviceModelInitialize(mesProduceSn, userName);
        return mesProduceSn;
    }

    private MesProduceSnPrintDataModel getModel(MesProduceSn produceSn, String no, String custPartNo, String factoryName) {
        MesProduceSnPrintDataModel mesProduceSnPrintDataModel = new MesProduceSnPrintDataModel();
        mesProduceSnPrintDataModel.setFactoryName(factoryName);
        mesProduceSnPrintDataModel.setPartNo(produceSn.getPartNo());
        mesProduceSnPrintDataModel.setPartName(produceSn.getPartName());
        mesProduceSnPrintDataModel.setNo(no);
        mesProduceSnPrintDataModel.setBarcode(produceSn.getProductSn());
        mesProduceSnPrintDataModel.setCustPartNo(custPartNo);
        mesProduceSnPrintDataModel.setProductDate(produceSn.getLotNo());
        return mesProduceSnPrintDataModel;
    }

}
